@@ -78,10 +78,10 @@ LibzipPlugin::~LibzipPlugin()
 bool LibzipPlugin::list()
 {
     qCDebug(ARK) << "Listing archive contents for:" << QFile::encodeName(filename());
-
+    setOperationMode(List);
     m_numberOfEntries = 0;
 
-    int errcode;
+    int errcode = 0;
     zip_error_t err;
 
     // Open archive.
@@ -124,8 +124,8 @@ bool LibzipPlugin::list()
 bool LibzipPlugin::addFiles(const QVector<Archive::Entry*> &files, const Archive::Entry *destination, const CompressionOptions& options, uint numberOfEntriesToAdd)
 {
     Q_UNUSED(numberOfEntriesToAdd)
-
-    int errcode;
+    setOperationMode(Add);
+    int errcode = 0;
     zip_error_t err;
 
     // Open archive.
@@ -391,7 +391,8 @@ bool LibzipPlugin::emitEntryForIndex(zip_t *archive, qlonglong index)
 
 bool LibzipPlugin::deleteFiles(const QVector<Archive::Entry*> &files)
 {
-    int errcode;
+    setOperationMode(Delete);
+    int errcode = 0;
     zip_error_t err;
 
     // Open archive.
@@ -436,7 +437,8 @@ bool LibzipPlugin::deleteFiles(const QVector<Archive::Entry*> &files)
 
 bool LibzipPlugin::addComment(const QString& comment)
 {
-    int errcode;
+    setOperationMode(Comment);
+    int errcode = 0;
     zip_error_t err;
 
     // Open archive.
@@ -465,8 +467,8 @@ bool LibzipPlugin::addComment(const QString& comment)
 bool LibzipPlugin::testArchive()
 {
     qCDebug(ARK) << "Testing archive";
-
-    int errcode;
+    setOperationMode(Test);
+    int errcode = 0;
     zip_error_t err;
 
     // Open archive performing extra consistency checks.
@@ -481,6 +483,10 @@ bool LibzipPlugin::testArchive()
     const int nofEntries = zip_get_num_entries(archive, 0);
     for (int i = 0; i < nofEntries; i++) {
 
+        if (QThread::currentThread()->isInterruptionRequested()) {
+            return false;
+        }
+
         // Get statistic for entry. Used to get entry size.
         zip_stat_t statBuffer;
         if (zip_stat_index(archive, i, 0, &statBuffer) != 0) {
@@ -489,13 +495,13 @@ bool LibzipPlugin::testArchive()
         }
 
         zip_file *zipFile = zip_fopen_index(archive, i, 0);
-        QScopedPointer<uchar> buf(new uchar[statBuffer.size]);
-        const int len = zip_fread(zipFile, buf.data(), statBuffer.size);
+        std::unique_ptr<uchar[]> buf(new uchar[statBuffer.size]);
+        const int len = zip_fread(zipFile, buf.get(), statBuffer.size);
         if (len == -1 || uint(len) != statBuffer.size) {
             qCCritical(ARK) << "Failed to read data for" << statBuffer.name;
             return false;
         }
-        if (statBuffer.crc != crc32(0, &buf.data()[0], len)) {
+        if (statBuffer.crc != crc32(0, &buf.get()[0], len)) {
             qCCritical(ARK) << "CRC check failed for" << statBuffer.name;
             return false;
         }
@@ -511,18 +517,28 @@ bool LibzipPlugin::testArchive()
 
 bool LibzipPlugin::doKill()
 {
-    qCDebug(ARK) << "Killing operation...";
+    QMutexLocker mutexLocker(&m_mutex);
+    switch (m_operationMode) {
+    case Add:
+    case Copy:
+    case Delete:
+    case Move:
+        return false;
+    default:
+        break;
+    }
+
     return true;
 }
 
 bool LibzipPlugin::extractFiles(const QVector<Archive::Entry*> &files, const QString& destinationDirectory, const ExtractionOptions& options)
 {
     qCDebug(ARK) << "Extracting files to:" << destinationDirectory;
-
+    setOperationMode(Extract);
     const bool extractAll = files.isEmpty();
     const bool removeRootNode = options.isDragAndDropEnabled();
 
-    int errcode;
+    int errcode = 0;
     zip_error_t err;
 
     // Open archive.
@@ -786,8 +802,8 @@ bool LibzipPlugin::extractEntry(zip_t *archive, const QString &entry, const QStr
 bool LibzipPlugin::moveFiles(const QVector<Archive::Entry*> &files, Archive::Entry *destination, const CompressionOptions &options)
 {
     Q_UNUSED(options)
-
-    int errcode;
+    setOperationMode(Move);
+    int errcode = 0;
     zip_error_t err;
 
     // Open archive.
@@ -836,8 +852,8 @@ bool LibzipPlugin::moveFiles(const QVector<Archive::Entry*> &files, Archive::Ent
 bool LibzipPlugin::copyFiles(const QVector<Archive::Entry*> &files, Archive::Entry *destination, const CompressionOptions &options)
 {
     Q_UNUSED(options)
-
-    int errcode;
+    setOperationMode(Copy);
+    int errcode = 0;
     zip_error_t err;
 
     // Open archive.
@@ -969,4 +985,10 @@ QString LibzipPlugin::permissionsToString(const mode_t &perm)
     return modeval;
 }
 
+void LibzipPlugin::setOperationMode(ReadWriteArchiveInterface::OperationMode operationMode)
+{
+    QMutexLocker mutexLocker(&m_mutex);
+    m_operationMode = operationMode;
+}
+
 #include "libzipplugin.moc"
